//  RW Mailstop
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./include/dollhouse/net/mailstop_util.hpp
// -Declarations of some baseline RW Mailstop utility functions

#pragma once

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

namespace rw::dollhouse::net {

constexpr auto droplog{"./dolldrops/dropmod.json"};  // dolldrops log's filename

//------------------------------------------------------------------------------
/** A simple data struct to help with local @c Dolldrop 's log entries
 */
struct Dropdata {
  string date;
  string name;
  string uri;
};

//------------------------------------------------------------------------------
/** Assign to a dolldrop data struct from a JSON object
 * @param[in] j
 * @param[out] d
 */
void from_json(Json const& j, Dropdata& d);

//------------------------------------------------------------------------------
/** Assign to a JSON object from a dolldrop data struct
 * @param[out] j
 * @param[in] d
 */
void to_json(Json& j, Dropdata const& d);

//------------------------------------------------------------------------------
/** Create a new logfile
 * @param file_nm
 */
void mk_logfile(std::string const& file_nm);

//------------------------------------------------------------------------------
/** Confirm the logfile exists
 */
void chk_logfile();

//------------------------------------------------------------------------------
/** Initialize a @a tagfile file
 * @param tagfile_nm
 * @note: initial tag number defaults to @a '000', so @a '001' will be the first
 * (that is, the @a next) tag number delivered by @c filetag() whenever this
 * function call is required
 */
void mk_tagfile(std::string const& file_nm);

//------------------------------------------------------------------------------
/** File-based scheme for doling out incremental numbers and (optionally) a
 * specified file-extension, provided together as a single tag string
 * @return the (next) file tag
 * @note: intended to serve as a filename suffix for differentiating subsequent
 * @c Dolldrop files
 * @note: this reads the count number from 'tagfile', then return the @a next
 * number filetag (also writing this number out to tagfile for next cycle)
 */
auto filetag() -> std::string;

//------------------------------------------------------------------------------
/** Assemble the basic, incrementally-numbered dolldrop filename
 * @return
 */
auto build_dropname() -> std::string;

//------------------------------------------------------------------------------
/** Confirm whether dropname is already listed in droplog, build new one if not
 * @param uri
 * @return
 */
auto chk_dropname(std::string const& uri) -> std::string;

//------------------------------------------------------------------------------
/** Read a dolldrop's header stream and locate it's @a field_nm tag, if present
 * @param[in] hdr_strm
 * @param[in] field_nm
 * @return the header's @a field_nm data, or empty string
 */
auto dropfield(std::stringstream& hdr_strm, std::string const& field_nm)
    -> std::string;

//------------------------------------------------------------------------------
/** Droplog-specific func to locate a JSON array element with matching @a uri
 * @param uri
 * @param json
 * @return iterator to JSON array element
 */
auto find_log_rec(std::string const& uri, Json& json) -> Json::iterator;

//------------------------------------------------------------------------------
/** Add a record into the droplog JSON object @a json_out
 * @param json_out
 * @param uri
 * @param tag
 * @param tagdat
 * @return true if record added successfully
 */
auto add_log_rec(Json&              json_out,
                 std::string const& uri,
                 std::string const& tag,
                 std::string const& tagdat) -> bool;

//------------------------------------------------------------------------------
/** Set a droplog record entry's specified @a tag with @a tagdat value
 * @param uri
 * @param tag
 * @param tagdat
 * @return true if tag set successfully
 * @note: @a uri parm is the search key to find specific record in log
 */
auto set_tag(std::string const& uri,
             std::string const& tag,
             std::string const& tagdat) -> bool;
/** Simple wrapper
 * @param uri
 * @param datedat
 * @return
 * @note: see @c set_tag()
 */
auto set_date(std::string const& uri, std::string const& datedat) -> bool;
/** Simple wrapper
 * @param uri
 * @param namedat
 * @return
 * @note: see @c set_tag()
 */
auto set_name(std::string const& uri, std::string const& namedat) -> bool;
/** Simple wrapper
 * @param uri
 * @param uridat
 * @return
 * @note: see @c set_tag()
 */
auto set_uri(std::string const& uri, std::string const& uridat) -> bool;

//------------------------------------------------------------------------------
/** Return a droplog record entry's specified @a tag value
 * @param uri
 * @param tag
 * @return
 * @note: @a uri parm is the search key to find specific record in log
 */
auto get_tag(std::string const& uri, std::string const& tag) -> std::string;
/** Simple wrapper
 * @param uri
 * @return
 * @note: see @c get_tag()
 */
auto get_date(std::string const& uri) -> std::string;
/** Simple wrapper
 * @param uri
 * @return
 * @note: see @c get_tag()
 */
auto get_name(std::string const& uri) -> std::string;

//------------------------------------------------------------------------------
/** Compare the dolldrop's header's current & stored 'last-modified' tags
 * @param uri
 * @param hdr
 * @return a @c std::tuple for use by dolldrop caller
 */
auto chk_droptimes(std::string const& uri, std::stringstream& hdr)
    -> std::tuple<bool, std::string>;

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dollhouse_lib.v220226
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
